#ifndef MODULE_INCLUDED
#define MODULE_INCLUDED

#include <common/node.hpp>
#include <rofi/voxel_graph.hpp>

#include <map>
#include <memory>
#include <string>

namespace rofi
{

class Module;

using module_ptr = std::shared_ptr<Module>;

class Module
{
    node_ptr node;
    voxel_graph_ptr parts;
    
    std::string name;
    std::string type;

    // uint64_t module_id;
    std::map<objectbase_ptr, uint64_t> component_ids;

    std::map<connector_ptr, std::string> connector_docks;

    void makeNodeTree();
    void setConnectorDocks();
    void setComponentIDs();

    Module(const Module &other);
    Module(const Module &&other);

    Module& operator=(const Module &other);
    Module& operator=(const Module &&other);

public:
    Module(voxel_graph_ptr _parts, std::string &_name, std::string &_type, std::map<objectbase_ptr, uint64_t> &_component_ids);
    Module(voxel_graph_ptr _parts, std::string &&_name, std::string &&_type, std::map<objectbase_ptr, uint64_t> &&_component_ids);
    ~Module() = default;

    static module_ptr copy(const module_ptr other_module);

    node_ptr getNode() const { return node; }
    voxel_graph_ptr getParts() const { return parts; }

    const std::string &getName() const { return name; }
    const std::string &getType() const { return type; }

    const std::map<objectbase_ptr, uint64_t> &getComponentIDs() const { return component_ids; }
    const std::map<connector_ptr, std::string> &getConnectorDocks() const { return connector_docks; }
    const uint64_t &getComponentID(objectbase_ptr component) const { return component_ids.at(component); }
    const std::string &getConnectorDock(connector_ptr connector) const { return connector_docks.at(connector); }
};

}

using module_ptr = std::shared_ptr<rofi::Module>;

#endif